<?php

namespace Drupal\web_push\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SettingsForm extends ConfigFormBase {

  public static $configId = 'web_push.settings';

  /**
   * Logger channel to add logs tracking.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $loggerChannel;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($config_factory);
    $this->loggerChannel = $this->logger('logger.channel.web_push');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'web_push_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::$configId
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::$configId);
    $form['options'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Default Options'),
    ];
    // TODO modify for not using seconds but days or something else.
    $form['options']['ttl'] = [
      '#type' => 'number',
      '#title' => $this->t('Time To Live'),
      '#description' => $this->t(
        "Specifying the number of seconds you want your push message to live on the push
        service before it's delivered"
      ),
      '#default_value' => $config->get('ttl') ?: 2419200,
    ];
    $form['options']['urgency'] = [
      '#type' => 'select',
      '#title' => $this->t('Urgency'),
      '#description' => $this->t(
        "Urgency indicates to the push service how important a message is to the user.
        This can be used by the push service to help conserve the battery life of a user's
        device by only waking up for important messages when battery is low."
      ),
      '#options' => [
        'very-low' => $this->t('very-low'),
        'low' => $this->t('low'),
        'normal' => $this->t('normal'),
        'high' => $this->t('high'),
      ],
      '#default_value' => $config->get('urgency') ?: 'normal',
    ];
    $form['options']['topic'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Topic'),
      '#description' => $this->t(
        'Topics are strings that can be used to replace a pending messages with a new message
        if they have matching topic names. This is useful in scenarios where multiple messages
        are sent while a device is offline, and you really only want a user to see the latest
        message when the device is turned on.'
      ),
      '#maxlength' => 32,
      '#default_value' => $config->get('topic') ?: '',
    ];
    $form['options']['batchSize'] = [
      '#type' => 'number',
      '#title' => $this->t('Batch size'),
      '#description' => $this->t(
        'If you send tens of thousands notifications at a time, you may get memory overflows
        due to how endpoints are called in Guzzle. In order to fix this, WebPush sends
        notifications in batches.'
      ),
      '#default_value' => $config->get('batchSize') ?: 1000,
    ];

    $form['displayBlock'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display request as a block in the page.'),
      '#default_value' => $config->get('displayBlock') ?: FALSE,
    ];

    $form['serviceWorker'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('List of other Service Worker'),
    ];
    $form['serviceWorker']['listRoute'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Other Service Worker Route'),
      '#description' => $this->t('Separate each route by a coma, exemple : pwa.service_worker_registration, advanced_pwa.serviceworker_file_data'),
      '#default_value' => $config->get('listRoute') ?: '',
    ];

    $form['actions'] = [
      '#type' => 'actions'
    ];
    $form['actions']['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save default configurations'),
    ];
    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getValue('listRoute')) {
      return;
    }
    $listRoute = explode(',', $form_state->getValue('listRoute'));
    $listeRouteError = [];

    foreach ($listRoute as $route) {
      try {
        Url::fromRoute(trim($route))->toString();
      }
      catch (\Exception $e) {
        $listeRouteError[] = $route;
      }
    }
    if ($listeRouteError) {
      $form_state->setErrorByName(
        'listRoute',
        $this->formatPlural(
          count($listeRouteError),
          '@route0 is not a valid Drupal Route',
          '@routes are not a valid Drupal Route',
          ['@route0' => $listeRouteError[0], '@routes' => implode(', ', $listeRouteError)]
        )
      );
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config(static::$configId);
    $config
      ->set('ttl', $form_state->getValue('ttl'))
      ->set('urgency', $form_state->getValue('urgency'))
      ->set('topic', $form_state->getValue('topic'))
      ->set('batchSize', $form_state->getValue('batchSize'))
      ->set('displayBlock', $form_state->getValue('displayBlock'))
      ->set('listRoute', $form_state->getValue('listRoute'))
      ->save();

    $this->loggerChannel->info($this->t('The userId : @user has modified the default configuration', ['@user' => $this->currentUser()->getAccount()->id()]));
    parent::submitForm($form, $form_state);
  }

}
