<?php

namespace Drupal\web_push\Service;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\web_push\Entity\Subscription;
use Drupal\web_push\Form\SettingsForm;
use Drupal\web_push\Service\AuthenticationHelper;
use Minishlink\WebPush\MessageSentReport;
use Minishlink\WebPush\WebPush;
use Minishlink\WebPush\Subscription as WebPushSubscription;
use Psr\Log\LoggerInterface;


class WebPushSender {

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The web_push config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Helper to get VAPID Keys.
   *
   * @var \Drupal\web_push\Service\AuthenticationHelper
   */
  protected $authenticationHelper;

  /**
   * Entity Storage to manage entity web_push_subscription.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * WebPush instance.
   *
   * @var \Minishlink\WebPush\WebPush
   */
  protected $webPush;

  /**
   * WebPushSender constructor.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(
    LoggerInterface $logger,
    ConfigFactoryInterface $config_factory,
    AuthenticationHelper $authentication_helper,
    EntityTypeManager $entity_type_manager,
  ) {
    $this->logger = $logger;
    $this->config = $config_factory->get(SettingsForm::$configId);
    $this->authenticationHelper = $authentication_helper;
    $this->entityStorage = $entity_type_manager->getStorage(('web_push_subscription'));
  }

  /**
   * Undocumented function
   *
   * @return void
   */
  protected function singletonWebPush() {
    if ($this->webPush) {
      return;
    }
    $auth = $this->authenticationHelper->getAuth();
    $options = [
      'TTL' => $this->config->get('ttl'),
      'urgency' => $this->config->get('urgency'),
      'topic' => $this->config->get('topic'),
      'batchSize' => (int) $this->config->get('batchSize')
    ];
    try {
      $this->webPush = new WebPush($auth, $options);
      $this->webPush->setDefaultOptions($options);
      // @see https://github.com/web-push-libs/web-push-php#reusing-vapid-headers.
      $this->webPush->setReuseVAPIDHeaders(true);
    }
    catch (\ErrorException $e) {
      $this->logger->error($e->getMessage());
    }
  }

  /**
   * Undocumented function
   *
   * @param Subscription $subscription
   * @param array $pushData
   *
   * @return array
   */
  protected function buildSubscription(Subscription $subscription, array $pushData) {
    $webSubscription['subscription'] = WebPushSubscription::create([
      'endpoint' => $subscription->getEndpoint(),
      'publicKey' => $subscription->getPublicKey(),
      'authToken' => $subscription->getToken()
    ]);
    $webSubscription['payload'] = Json::encode($pushData);
    return $webSubscription;
  }

  protected function prepareSubscriptions(array $pushData) {
    $listSubscriptions = $this->entityStorage->loadMultiple();
    foreach ($listSubscriptions as $subscription) {
      $webSubscription = $this->buildSubscription($subscription, $pushData);
      try {
        $this->webPush->queueNotification(
          $webSubscription['subscription'],
          $webSubscription['payload']
        );
      }
      catch (\ErrorException $e) {
        $this->logger->error($e->getMessage());
      }
    }
  }

  /**
   * Manage action on result event.
   *
   * @param MessageSentReport $report
   *
   * @return void
   */
  protected function manageReport(MessageSentReport $report) {
    $endpoint = $report->getEndpoint();
    if ($report->isSuccess()) {
      $this->logger->notice("[v] Message sent successfully for subscription {$endpoint}.");
      return;
    }
    $this->logger->notice("[x] Message failed to sent for subscription {$endpoint}: {$report->getReason()}");
    if ($report->isSubscriptionExpired()) {
      // Get the subscription to delete.
      $subscriptionList = $this->entityStorage->loadByProperties(['endpoint' => $endpoint]);
      foreach($subscriptionList as $subscription) {
        $subscription->delete();
      }
    }
  }

  public function sendNotification(array $pushData) {
    $this->singletonWebPush();
    $this->prepareSubscriptions($pushData);

    // Send the queued items by prepareSubscriptions().
    // And check message for each notifications.
    // @see https://github.com/web-push-libs/web-push-php#server-errors.
    /** @var \Minishlink\WebPush\MessageSentReport $report */
    foreach ($this->webPush->flush() as $report) {
      $this->manageReport($report);
    }
  }

}
